##
## copy from buzz.py to support douban timeline
## Sun Ning <classicning@gmail.com>
##

import re, urllib
import network, util, json
from oauth import oauth

OAUTH_CONSUMER_KEY = "03082894afa118cf091879be0ee34a6d"
OAUTH_CONSUMER_SECRET = "85b427ca2f4ba3d1"

PROTOCOL_INFO = {
  "name": "Douban",
  "version": "1.0",

  "config": [
    "color",
    "receive_enabled",
    "send_enabled",
    "username",
    "access_token",
    "private:secret_token",
  ],

  "authtype": "oauth1a",
  "color": "#339900",

  "features": [
    "receive",
    "reply",
    "delete",
    "send",
### due to upstream functionality limits, 
### delete is not enabled in this version
### CurlDownloader does not support DELETE method
#    "delete",
    "user_messages",
    "retweet",
  ],

  "default_streams": [
    "receive",
  ],
}

USER_ID_URL_TEMPLATE = "http://api.douban.com/people/(.*)"
#MSG_ID_URL_TEMPLATE = "http://api.douban.com/miniblog/(.*)"
URL_SELF_GET = "http://api.douban.com/people/{0}/miniblog"
URL_CONTACTS_GET = "http://api.douban.com/people/{0}/miniblog/contacts"
URL_POST = "http://api.douban.com/miniblog/saying"
URL_DELETE = "http://api.douban.com/miniblog/{0}"
URL_SAYING_COMMENT = "http://api.douban.com/miniblog/{0}/comments"
URL_RECOMMANDATION_COMMENT = "http://api.douban.com/recommendation/{0}/comments"

POST_BODY_TEMPLATE = """<?xml version='1.0' encoding='UTF-8'?>
<entry xmlns:ns0="http://www.w3.org/2005/Atom" xmlns:db="http://www.douban.com/xmlns/">
<content>{0}</content>
</entry>
"""

class Client:
  def __init__(self, acct):
    self.account = acct
    self.sigmethod = oauth.OAuthSignatureMethod_HMAC_SHA1()
    self.consumer = oauth.OAuthConsumer(OAUTH_CONSUMER_KEY, OAUTH_CONSUMER_SECRET)
    self.token = oauth.OAuthToken(acct["access_token"], acct["secret_token"])

  def _get_user_id_from_uri(self, uri):
    user_id = re.search(USER_ID_URL_TEMPLATE, uri).group(1)
    return user_id

  def _get_msg_id_from_uri(self, uri):
    msg_id = re.search(MSG_ID_URL_TEMPLATE, uri).group(1)
    return msg_id
    
  def _user(self, user):
    ## default user avatar
    icon = "http://img3.douban.com/icon/user_normal.jpg"

    ## if user has a custom avatar
    for link in user["link"]:
      if link["@rel"] == "icon":
        icon = link["@href"]
        break

    user_id = self._get_user_id_from_uri(user["uri"]["$t"])

    return {
        "name": user["name"]["$t"],
        "nick": user_id,
        "id": user_id,
        "image": icon,
        "url": user["uri"]["$t"],
        "is_me": user_id == self.account["user_id"],
    }

  def _message(self, data, author=None):
    ## a work around to handle contact_message and user_message
    ## because there is no author property included in message object
    ## if the data returned by a user_message request
    if "author" in data:
      author = data["author"]

    m = {
        "mid": data["id"]["$t"],
        "service": "douban",
        "account": self.account["id"],
        "time": util.parsetime(data["published"]["$t"]),
        "url": data["id"]["$t"],
        "source": "douban",
        "sender": self._user(author),
        "category": data['category']['@term']
    }

    m["text"] = data["content"]["$t"]

    if "db:attribute" in data:
      for dbattr in data["db:attribute"]:
        if dbattr['@name'] == 'comment':
          m["text"] += " "
          m["text"] += dbattr['$t']

    m["html"] = m["text"]
    m["content"] = m["text"]

    m["images"] = []
    if "link" in data:
      image_src = None
      image_url = None

      for link in data["link"]:
        if link["@rel"] == "image":
          image_src = link["@href"]
        if link["@rel"] == "related":
          image_url = link["@href"].replace("api", "www", 1)
          m["related"] = link["@href"]

      if image_src and image_url:
        m["images"].append(dict(src=image_src, url=image_url))

    return m

  def _post(self, url, data):
    request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, self.token,
        http_method="POST", http_url=url)
    
    request.sign_request(self.sigmethod, self.consumer, self.token)
    oauthheader = request.to_header(realm="http://api.douban.com")['Authorization']
    data = network.Download(url, None, True,
        header=['Authorization:'+oauthheader, "Content-Type: application/atom+xml"], body=data)
    return []

  def _delete(self, url):
    request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, self.token,
        http_method="DELETE", http_url=url)
    request.sign_request(self.sigmethod, self.consumer, self.token)
    network.Download(request.to_url(), None, "DELETE")
    return []

  def _get(self, url):
    request = oauth.OAuthRequest.from_consumer_and_token(self.consumer, self.token,
        http_method="GET", http_url=url, parameters={"alt":"json", "max-results":20})
    request.sign_request(self.sigmethod, self.consumer, self.token)

    data = network.Download(request.to_url(), {})
    
    data = json.loads(data.get_string())

    if data:
      messages = data["entry"]
      msgs = []
      for entry in messages:
        msg = self._message(entry, data["author"])
        msgs.append(msg)
      return msgs
    else: 
      return []

  def __call__(self, opname, **args):
    return getattr(self, opname)(**args)

  def receive(self, count=util.COUNT, since=None):
    return self._get(URL_CONTACTS_GET.format(self.account['user_id']))

  def user_messages(self, id, count=util.COUNT, since=None):
    return self._get(URL_SELF_GET.format(id))

  def send(self, message):
    text = POST_BODY_TEMPLATE.format(message)
    return self._post(URL_POST, text)

  def delete(self, message):
    msg_id = self._get_msg_id_from_uri(message['mid'])
    return self._delete(URL_DELETE.format(msg_id))

  def send_thread(self, message, target):
    text = POST_BODY_TEMPLATE.format(message)
    ## You can only post comment on saying and recommendation
    ## But they are under different url !! WTF!!
    url = target['mid']+"/comments"
    if 'category' in target:
      if target['category'].endswith("recommendation"):
        url = target['related']+"/comments"
    return self._post(url, text)

